/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.cvut.kbss.dao;

import cz.cvut.kbss.bodm.DomainEntity;
import cz.cvut.kbss.jpautils.JPAUtility;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.ParameterExpression;
import javax.persistence.criteria.Root;

/**
 *
 * @author user
 */
public class GenericDAOImpl implements GenericDAOIface{

    private static GenericDAOIface genericDAO;

    public static GenericDAOIface getGenericDAO() {
        if(genericDAO == null){
            genericDAO = new GenericDAOImpl();
        }
        return genericDAO;
    }
    
    private GenericDAOImpl() {
    }

    protected <ENTITY> List<ENTITY> getAll(Class<ENTITY> clazz, EntityManager em) {
        return (List<ENTITY>)em.createQuery("select e from " + clazz.getSimpleName() + " e").getResultList();
    }
    
    public <ENTITY> List<ENTITY> getAll(Class<ENTITY> clazz) {
        EntityManager em = JPAUtility.createNewEntityManager();
        return getAll(clazz, em);
    }

    public <ENTITY extends DomainEntity> ENTITY saveOrUpdate(ENTITY inst) {
        EntityManager em = JPAUtility.createNewEntityManager();
        em.getTransaction().begin();
        if(inst.getId() == null){
            em.persist(inst);
        }else{
            em.merge(inst);
        }
        em.getTransaction().commit();
        em.close();
        return inst;
    }
    
    public <ENTITY> ENTITY saveEntity(ENTITY entity){
        EntityManager em = JPAUtility.createNewEntityManager();
        em.getTransaction().begin();
        em.persist(entity);
        em.getTransaction().commit();
        return entity;
    }
    
    public <ENTITY> ENTITY mergeEntity(ENTITY entity){
        EntityManager em = JPAUtility.createNewEntityManager();
        em.getTransaction().begin();
        ENTITY ret = em.merge(entity);
        em.getTransaction().commit();
        return ret;
    }
    
    public <ENTITY> void removeAll(Class<ENTITY> clazz){
        EntityManager em = JPAUtility.createNewEntityManager();
        em.getTransaction().begin();
        List<ENTITY> entities = this.getAll(clazz, em);
        for(ENTITY entity : entities){
            em.remove(entity);
        }
        em.getTransaction().commit();
    } 
    
    public <ENTITY> ENTITY findBy(Class<ENTITY> clazz, String attrName, String attrVal){
        EntityManager em = JPAUtility.createNewEntityManager();
//        List<ENTITY> resultList = (List<ENTITY>)em.createQuery("select e from " + clazz.getSimpleName() + " e WHERE e." + attrName + " = " + attrVal).getResultList();
//        return resultList.get(0);
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<ENTITY> cq = cb.createQuery(clazz);
        Root<ENTITY> c = cq.from(clazz);
        ParameterExpression<String> param = cb.parameter(String.class);
        cq.select(c).where(cb.equal(c.get(attrName), param));
        
        TypedQuery<ENTITY> q = em.createQuery(cq);
//        TypedQuery<ENTITY> q = em.createQuery("SELECT e FROM " + clazz.getSimpleName() + " e WHERE :attr = :val", clazz);
        q.setParameter(param, attrVal);
//        q.setParameter("val", attrVal);
        List<ENTITY> list = q.getResultList();
        if(list == null || list.isEmpty()){
            return null;
        }
        if(list.size() > 1){
            Logger.getLogger(GenericDAOImpl.class.getName()).log(Level.WARNING, "A query with unique attribute returned not unique result!");
            return null;
        }
        return list.get(0);
    }

}
